#include <CImg.h>
#include <gflags/gflags.h>
#include <glog/logging.h>
#include <stdio.h>
#include <stdlib.h>
#include <util/assert.h>
#include <util/directory.h>

DEFINE_string(directory, 
	      "./", 
	      "The directory where the images to be stitched are located.");
DEFINE_string(image_type, "jpg", "The type of image (jpg, gif, etc).");
DEFINE_int32(zoom_level, 4, "The zoom level for the panorama.");
DEFINE_int32(tile_size, -1, "The size of each of the panorama tiles.");
DEFINE_string(output_file, "panorama.jpg", "The name of the output file.");
DEFINE_int32(resize_percentage, 80, "The percentage to scale the output by before saving.");

using namespace cimg_library;

int main(int argc, char** argv) {
  google::ParseCommandLineFlags(&argc, &argv, true);
  google::InitGoogleLogging(argv[0]);
 
  ASSERT_LTE(FLAGS_zoom_level, 5);
  ASSERT_GTE(FLAGS_zoom_level, 0);
 
  // Load all of the images.
  vector<string> files = slib::util::Directory::GetDirectoryContents(FLAGS_directory, 
								     FLAGS_image_type);

  LOG(INFO) << "Found a total of " << files.size() 
	    << " files in directory " << FLAGS_directory;

  if (FLAGS_tile_size < 0) {
    string path = FLAGS_directory + "/0.0.jpg";
    CImg<unsigned char> dummy(path.c_str());
    FLAGS_tile_size = dummy.width();
    LOG(INFO) << "Automatically set tile size to: " << FLAGS_tile_size;
  }

  // This is a static array that maps the zoom level to the number of
  // x/y positions there are.
  const int x_bounds[] = {1, 2, 4, 6, 13, 26};
  const int y_bounds[] = {1, 1, 2, 3, 7, 13};
 
  CImg<unsigned char> final(FLAGS_tile_size * x_bounds[FLAGS_zoom_level],
			    FLAGS_tile_size * y_bounds[FLAGS_zoom_level],
			    1, 3, 0);
  // Copy tiles to the final image.
  for (int y = 0; y < y_bounds[FLAGS_zoom_level]; y++) {
    for (int x = 0; x < x_bounds[FLAGS_zoom_level]; x++) {
      char path[1024];
      sprintf(path, "%s/%d.%d.jpg", FLAGS_directory.c_str(), y, x);
      
      CImg<unsigned char> image(path);
      final.draw_image(x * FLAGS_tile_size, y * FLAGS_tile_size, image);
    }
  }

  LOG(INFO) << "Saving file to: " << FLAGS_output_file;
  if (FLAGS_tile_size >= 512) {
    final
      .resize(-FLAGS_resize_percentage, 
	      -FLAGS_resize_percentage, 
	      -FLAGS_resize_percentage, 
	      final.spectrum(), 5)
      .save_jpeg(string(FLAGS_output_file).c_str(), 100);
  } else {
    final.save_jpeg(string(FLAGS_output_file).c_str(), 100);
  }
}
