function model = testTraining(test_panorama, input_file, panoramas_base, num_positives, num_negatives, scale, solver_type, bounds, panoramas)

if (nargin < 9)
    fprintf(1, 'No panoramas specified. Loading from input file.\n');
    panoramas = loadTrainingData(input_file, panoramas_base, scale);
end

panoramas = applyPanoramaBounds(panoramas, bounds);

fprintf(1, 'Generating positive examples.\n');
[positives pmap] = generatePositiveExamples(panoramas, num_positives);
fprintf(1, 'Generating negative examples.\n');
[negatives nmap] = generateNegativeExamples(panoramas, num_negatives);

%figure; imagesc(pmap); title 'Positive Examples';
%figure; imagesc(nmap); title 'Negative Examples';

training_data = [positives;negatives];
labels = [ones(size(positives,1),1); -ones(size(negatives,1),1)];

fprintf('Extracting features from input image.\n');
[f features_string] = getFeatures(imresize(imread([panoramas_base '/' test_panorama]), scale));
features = reshape(f, size(f,1) * size(f,2), size(f,3));

predictions = zeros(size(features, 1), 1);
use_svm = true;
if (use_svm)
    fprintf(1, 'Training model (SVM).\n');
    [type type_string] = getType(solver_type);
    model = train(labels, sparse(training_data), ['-c 10 -s ' type ' -e 0.01']);
    clear training_data labels positives negatives;

    % Run model against the panorama.
    fprintf(1, 'Computing predictions from model.\n');
    [labels, accuracy, prob_estimates] = predict(randi([0,1],size(features,1),1), sparse(features), model, '-b 1');
    
    w = model.w;
    for i=1:size(predictions,1)
        predictions(i) = sqrt(sum((features(i,:)-w).^2));
    end
    predictions = (predictions - min(predictions(:))) / (max(predictions(:)) - min(predictions(:)));
else
    fprintf('Training model (adaboost).\n');
    type_string = '_.adaboost._';
    weak_learner = tree_node_w(3);
    [Learners Weights] = RealAdaBoost(weak_learner, training_data', labels', 100);
    labels = sign(Classify(Learners, Weights, features'))';
end

% Create visualization.
D = datestr(now, 'mm.dd-HH.MM.SS');
output_file = [D '.' type_string '.' num2str(num_positives) '.' num2str(num_negatives) '.' features_string '.' test_panorama];
if (use_svm) 
    makeSaliencyOverlay(imresize(imread([panoramas_base '/' test_panorama]), scale), labels, [output_file '.labels']);
    makePredictionOverlay(imresize(imread([panoramas_base '/' test_panorama]), scale), predictions, [output_file '.predictions']);
else
    makeSaliencyOverlay(imresize(imread([panoramas_base '/' test_panorama]), scale), labels, [output_file '.labels']);
end
imwrite(mean(f,3), ['results/' output_file '.features.png']);