#include <CImg.h>
#include <common/types.h>
#include <gflags/gflags.h>
#include <glog/logging.h>
#include <map>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <util/assert.h>
#include <vector>

DEFINE_string(input_file, "output.dat", "The input filename.");
DEFINE_string(panorama_base_directory, "", "The base location for the panoramas.");
DEFINE_string(output_directory, ".", "The directory where the images will be written.");
DEFINE_double(alpha, 0.3, "The opacity to render each box as.");

using namespace cimg_library;
using std::map;
using std::string;
using std::vector;

struct Annotations {
  int32 x;
  int32 y;
  int32 width;
  int32 height;
};

typedef map<string, vector<Annotations> > TrainingData;

int LoadTrainingData(const string& output_file, TrainingData* training_data) {
  FILE* fid = fopen(output_file.c_str(), "r");
  if (!fid) {
    LOG(WARNING) << "Could not open output file: " << output_file;
    return 1;
  }

  int32 N;
  fscanf(fid, "%ld", &N);

  // Read in the list of panoids.
  vector<string> panoids;
  for (int32 i = 0; i < N; i++) {
    char buffer[1024];
    fscanf(fid, "%s", buffer);
    panoids.push_back(string(buffer));
  }

  // Read in the annotations.
  for (int32 i = 0; i < N; i++) {
    int32 M;
    fscanf(fid, "%ld", &M);
    vector<Annotations> annotations;
    for (int32 j = 0; j < M; j++) {
      int32 x, y, width, height;
      fscanf(fid, "%ld\n", &x);
      fscanf(fid, "%ld\n", &y);
      fscanf(fid, "%ld\n", &width);
      fscanf(fid, "%ld\n", &height);

      Annotations annotation;
      annotation.x = x;
      annotation.y = y;
      annotation.width = width;
      annotation.height = height;

      annotations.push_back(annotation);
    }
    ASSERT_LT(i, static_cast<int32>(panoids.size()));
    training_data->insert(make_pair(panoids[i], annotations));
  }

  fclose(fid);

  return 0;
}

int main(int argc, char** argv) {
  google::ParseCommandLineFlags(&argc, &argv, true);
  google::InitGoogleLogging(argv[0]);

  TrainingData training_data;
  if (LoadTrainingData(FLAGS_input_file, &training_data) != 0) {
    LOG(ERROR) << "Could not load data from input file: " << FLAGS_input_file;
    return 1;
  }
  
  LOG(INFO) << "Found a total of " << training_data.size() << " panoids.";

  for (TrainingData::const_iterator iter = training_data.begin(); iter != training_data.end(); iter++) {
    string image_file = FLAGS_panorama_base_directory + "/" + (*iter).first;
    CImg<uint8> output_image(image_file.c_str());
    for (int32 i = 0; i < static_cast<int32>((*iter).second.size()); i++) {
      const int32 x = (*iter).second[i].x;
      const int32 y = (*iter).second[i].y;
      const int32 width = (*iter).second[i].width;
      const int32 height = (*iter).second[i].height;
      const uint8 color[3] = { 255, 0, 0 };
      output_image.draw_rectangle(x, y, x + width, y+height, color, FLAGS_alpha);
    }

    LOG(INFO) << "Writing annotated image for: " << (*iter).first;
    string output_file = FLAGS_output_directory + "/" + (*iter).first + "_annotations.png";
    LOG(INFO) << "Writing to file: " << output_file;
    output_image.save_png(output_file.c_str());
  }
  
  return 0;
}
